use std::env;
use std::fs;
use serde::{Deserialize, Serialize};
use regex::Regex;

#[derive(Serialize, Deserialize, Debug)]
struct BuuTrack {
    #[serde(rename = "$format")]
    format: i32,
    Markers: Vec<i32>,
    Signatures: Vec<Signature>,
    Sections: Vec<serde_json::Value>,
    Comments: Vec<serde_json::Value>,
    Ambiances: Vec<serde_json::Value>,
    AudioFilePath: String,
    StartBeat: i32,
    EndBeat: i32,
    VideoStartTime: String,
    Volume: f32,
    PreviewEntry: f32,
    PreviewLoopStart: f32,
    PreviewLoopEnd: f32,
    FadeInDuration: f32,
    FadeOutDuration: f32,
    FadeStartBeat: f32,
    FadeEndBeat: f32,
    UseFadeStartBeat: bool,
    UseFadeEndBeat: bool,
    FadeInType: i32,
    FadeOutType: i32,
}

#[derive(Serialize, Deserialize, Debug)]
struct Signature {
    Comment: String,
    Marker: f32,
    Beat: i32,
}

fn parse_drum_trk(content: &str) -> Result<BuuTrack, Box<dyn std::error::Error>> {
    // Extract markers
    let re_markers = Regex::new(r"\{\s*VAL\s*=\s*(\d+)\s*\}")?;
    let markers: Vec<i32> = re_markers
        .captures_iter(content)
        .map(|cap| cap[1].parse().unwrap())
        .collect();

    // Extract signature data
    let re_signature = Regex::new(r#"MusicSignature\s*=\s*\{\s*beats\s*=\s*(\d+),\s*marker\s*=\s*([0-9.]+),\s*comment\s*=\s*"(.*?)"\s*\}"#)?;
    let mut signatures = Vec::new();
    
    for cap in re_signature.captures_iter(content) {
        let beat = cap[1].parse()?;
        let marker = cap[2].parse()?;
        let comment = cap[3].to_string();
        
        signatures.push(Signature {
            Comment: comment,
            Marker: marker,
            Beat: beat,
        });
    }

    // Extract start and end beats
    let re_beats = Regex::new(r"startBeat\s*=\s*(\d+),\s*endBeat\s*=\s*(\d+)")?;
    let (start_beat, end_beat) = if let Some(cap) = re_beats.captures(content) {
        (cap[1].parse()?, cap[2].parse()?)
    } else {
        (0, 0)
    };

    // Extract video start time
    let re_video_time = Regex::new(r"videoStartTime\s*=\s*([0-9.]+)")?;
    let video_time = if let Some(cap) = re_video_time.captures(content) {
        cap[1].parse::<f32>()?
    } else {
        0.0
    };

    // Format video time as HH:MM:SS
    let hours = (video_time / 3600.0) as i32;
    let minutes = ((video_time % 3600.0) / 60.0) as i32;
    let seconds = (video_time % 60.0) as i32;
    let video_time_str = format!("{:02}:{:02}:{:02}", hours, minutes, seconds);

    // Extract volume
    let re_volume = Regex::new(r"volume\s*=\s*([0-9.]+)")?;
    let volume = if let Some(cap) = re_volume.captures(content) {
        cap[1].parse()?
    } else {
        0.0
    };

    // Extract preview values
    let re_preview = Regex::new(r"previewEntry\s*=\s*([0-9.]+),\s*previewLoopStart\s*=\s*([0-9.]+),\s*previewLoopEnd\s*=\s*([0-9.]+)")?;
    let (preview_entry, preview_loop_start, preview_loop_end) = if let Some(cap) = re_preview.captures(content) {
        (cap[1].parse()?, cap[2].parse()?, cap[3].parse()?)
    } else {
        (0.0, 0.0, 0.0)
    };

    // Construct the BuuTrack
    let buu_track = BuuTrack {
        format: 0,
        Markers: markers,
        Signatures: signatures,
        Sections: Vec::new(),
        Comments: Vec::new(),
        Ambiances: Vec::new(),
        AudioFilePath: "maps\\Drum\\audio\\Drum.wav".to_string(), // Default path
        StartBeat: start_beat,
        EndBeat: end_beat,
        VideoStartTime: video_time_str,
        Volume: volume,
        PreviewEntry: preview_entry,
        PreviewLoopStart: preview_loop_start,
        PreviewLoopEnd: preview_loop_end,
        FadeInDuration: 0.0,
        FadeOutDuration: 0.0,
        FadeStartBeat: 0.0,
        FadeEndBeat: 0.0,
        UseFadeStartBeat: false,
        UseFadeEndBeat: false,
        FadeInType: 0,
        FadeOutType: 0,
    };

    Ok(buu_track)
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let args: Vec<String> = env::args().collect();
    
    if args.len() != 3 {
        eprintln!("Usage: {} <input_drum.trk> <output_6buu.trk>", args[0]);
        std::process::exit(1);
    }
    
    let input_path = &args[1];
    let output_path = &args[2];
    
    // Read the input file
    let content = fs::read_to_string(input_path)
        .map_err(|e| format!("Failed to read input file: {}", e))?;
    
    // Parse the Drum.trk content
    let buu_track = parse_drum_trk(&content)?;
    
    // Serialize to JSON
    let json = serde_json::to_string_pretty(&buu_track)?;
    
    // Write to output file
    fs::write(output_path, json)
        .map_err(|e| format!("Failed to write output file: {}", e))?;
    
    println!("Successfully converted {} to {}", input_path, output_path);
    
    Ok(())
}